@@ -22,7 +22,7 @@ To set proxy, please set env var HTTP_PROXY and HTTPS_PROXY (eg export HTTPS_PRO
 Usage:
     qsv fetch [options] [--http-header <k:v>...] [<column>] [<input>]
 
-fetch options:
+Fetch options:
     -c, --new-column <name>    Put the fetched values in a new column instead.
     --jql <selector>           Apply jql selector to API returned JSON value.
     --rate-limit <qps>         Rate Limit in Queries Per Second. [default: 5]
@@ -304,7 +304,7 @@ fn apply_jql(json: &str, selectors: &str) -> Result<String> {
             Ok(valid_json) => {
                 // Walk through the JSON content with the provided selectors as
                 // input.
-                match walker(&valid_json, Some(selectors)) {
+                match walker(&valid_json, selectors) {
                     Ok(selection) => {
                         fn get_value_string(v: &Value) -> String {
                             if v.is_null() {
