@@ -18,7 +18,7 @@ use crate::CliResult;
 use dateparser::DateTimeUtc;
 use serde::Deserialize;
 
-use self::FieldType::{TDate, TFloat, TInteger, TNull, TUnicode, TUnknown};
+use self::FieldType::{TDate, TFloat, TInteger, TNull, TString, TUnknown};
 
 static USAGE: &str = "
 Computes basic statistics on CSV data.
@@ -30,7 +30,7 @@ statistics are reported for *every* column in the CSV data: sum, min/max values,
 min/max length, mean, stddev & variance. The default set of statistics corresponds to
 statistics that can be computed efficiently on a stream of data (i.e., constant memory).
 
-The data type of each column is also inferred (Unknown, NULL, Integer, Unicode,
+The data type of each column is also inferred (Unknown, NULL, Integer, String,
 Float and Date). The date formats recognized can be found at
 https://docs.rs/dateparser/0.1.6/dateparser/#accepted-date-formats.
 
@@ -78,20 +78,20 @@ Common options:
 ";
 
 #[derive(Clone, Deserialize)]
-struct Args {
-    arg_input: Option<String>,
-    flag_select: SelectColumns,
-    flag_everything: bool,
-    flag_mode: bool,
-    flag_cardinality: bool,
-    flag_median: bool,
-    flag_quartiles: bool,
-    flag_nulls: bool,
-    flag_nullcount: bool,
-    flag_jobs: isize,
-    flag_output: Option<String>,
-    flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
+pub struct Args {
+    pub arg_input: Option<String>,
+    pub flag_select: SelectColumns,
+    pub flag_everything: bool,
+    pub flag_mode: bool,
+    pub flag_cardinality: bool,
+    pub flag_median: bool,
+    pub flag_quartiles: bool,
+    pub flag_nulls: bool,
+    pub flag_nullcount: bool,
+    pub flag_jobs: isize,
+    pub flag_output: Option<String>,
+    pub flag_no_headers: bool,
+    pub flag_delimiter: Option<Delimiter>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -126,14 +126,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 }
 
 impl Args {
-    fn sequential_stats(&self) -> CliResult<(csv::ByteRecord, Vec<Stats>)> {
+    pub fn sequential_stats(&self) -> CliResult<(csv::ByteRecord, Vec<Stats>)> {
         let mut rdr = self.rconfig().reader()?;
         let (headers, sel) = self.sel_headers(&mut rdr)?;
         let stats = self.compute(&sel, rdr.byte_records())?;
         Ok((headers, stats))
     }
 
-    fn parallel_stats(
+    pub fn parallel_stats(
         &self,
         idx: Indexed<fs::File, fs::File>,
     ) -> CliResult<(csv::ByteRecord, Vec<Stats>)> {
@@ -164,7 +164,7 @@ impl Args {
         Ok((headers, merge_all(recv.iter()).unwrap_or_else(Vec::new)))
     }
 
-    fn stats_to_records(&self, stats: Vec<Stats>) -> Vec<csv::StringRecord> {
+    pub fn stats_to_records(&self, stats: Vec<Stats>) -> Vec<csv::StringRecord> {
         let mut records: Vec<_> = repeat(csv::StringRecord::new()).take(stats.len()).collect();
         let pool = ThreadPool::new(self.njobs());
         let mut results = vec![];
@@ -181,6 +181,7 @@ impl Args {
         records
     }
 
+    #[inline]
     fn compute<I>(&self, sel: &Selection, it: I) -> CliResult<Vec<Stats>>
     where
         I: Iterator<Item = csv::Result<csv::ByteRecord>>,
@@ -189,7 +190,11 @@ impl Args {
         for row in it {
             let row = row?;
             for (i, field) in sel.select(&row).enumerate() {
-                stats[i].add(field);
+                unsafe {
+                    // we use unsafe here so we skip unnecessary
+                    // bounds checking for this hot loop
+                    stats.get_unchecked_mut(i).add(field);
+                }
             }
         }
         Ok(stats)
@@ -204,7 +209,7 @@ impl Args {
         Ok((csv::ByteRecord::from_iter(sel.select(&headers)), sel))
     }
 
-    fn rconfig(&self) -> Config {
+    pub fn rconfig(&self) -> Config {
         Config::new(&self.arg_input)
             .delimiter(self.flag_delimiter)
             .no_headers(self.flag_no_headers)
@@ -221,6 +226,7 @@ impl Args {
         }
     }
 
+    #[inline]
     fn new_stats(&self, record_len: usize) -> Vec<Stats> {
         repeat(Stats::new(WhichStats {
             include_nulls: self.flag_nulls,
@@ -237,7 +243,7 @@ impl Args {
         .collect()
     }
 
-    fn stat_headers(&self) -> csv::StringRecord {
+    pub fn stat_headers(&self) -> csv::StringRecord {
         let mut fields = vec![
             "field",
             "type",
@@ -296,7 +302,7 @@ impl Commute for WhichStats {
 }
 
 #[derive(Clone)]
-struct Stats {
+pub struct Stats {
     typ: FieldType,
     sum: Option<TypedSum>,
     minmax: Option<TypedMinMax>,
@@ -344,6 +350,7 @@ impl Stats {
     }
 
     #[allow(clippy::option_map_unit_fn)]
+    #[inline]
     fn add(&mut self, sample: &[u8]) {
         let sample_type = FieldType::from_sample(sample);
         self.typ.merge(sample_type);
@@ -370,7 +377,7 @@ impl Stats {
                     };
                 }
             }
-            TUnicode => {}
+            TString => {}
             TFloat | TInteger => {
                 if sample_type.is_null() {
                     if self.which.include_nulls {
@@ -396,7 +403,7 @@ impl Stats {
     }
 
     #[allow(clippy::wrong_self_convention)]
-    fn to_record(&mut self) -> csv::StringRecord {
+    pub fn to_record(&mut self) -> csv::StringRecord {
         let typ = self.typ;
         let mut pieces = vec![];
         let empty = || "".to_owned();
@@ -530,18 +537,18 @@ impl Commute for Stats {
     }
 }
 
-#[derive(Clone, Copy, PartialEq)]
-enum FieldType {
+#[derive(Clone, Copy, PartialEq, Eq, Hash)]
+pub enum FieldType {
     TUnknown,
     TNull,
-    TUnicode,
+    TString,
     TFloat,
     TInteger,
     TDate,
 }
 
 impl FieldType {
-    fn from_sample(sample: &[u8]) -> FieldType {
+    pub fn from_sample(sample: &[u8]) -> FieldType {
         if sample.is_empty() {
             return TNull;
         }
@@ -558,7 +565,7 @@ impl FieldType {
         if string.parse::<DateTimeUtc>().is_ok() {
             return TDate;
         }
-        TUnicode
+        TString
     }
 
     fn is_number(&self) -> bool {
@@ -573,7 +580,7 @@ impl FieldType {
 impl Commute for FieldType {
     fn merge(&mut self, other: FieldType) {
         *self = match (*self, other) {
-            (TUnicode, TUnicode) => TUnicode,
+            (TString, TString) => TString,
             (TFloat, TFloat) => TFloat,
             (TInteger, TInteger) => TInteger,
             (TDate, TDate) => TDate,
@@ -586,10 +593,10 @@ impl Commute for FieldType {
             // when using unixtime format can degrade to int/floats.
             (TInteger, TDate) | (TDate, TInteger) => TInteger,
             (TFloat, TDate) | (TDate, TFloat) => TFloat,
-            // Numbers/dates can degrade to Unicode strings.
-            (TUnicode, TFloat) | (TFloat, TUnicode) => TUnicode,
-            (TUnicode, TInteger) | (TInteger, TUnicode) => TUnicode,
-            (TUnicode, TDate) | (TDate, TUnicode) => TUnicode,
+            // Numbers/dates can degrade to unicode Strings.
+            (TString, TFloat) | (TFloat, TString) => TString,
+            (TString, TInteger) | (TInteger, TString) => TString,
+            (TString, TDate) | (TDate, TString) => TString,
         };
     }
 }
@@ -608,7 +615,20 @@ impl fmt::Display for FieldType {
         match *self {
             TUnknown => write!(f, "Unknown"),
             TNull => write!(f, "NULL"),
-            TUnicode => write!(f, "Unicode"),
+            TString => write!(f, "String"),
+            TFloat => write!(f, "Float"),
+            TInteger => write!(f, "Integer"),
+            TDate => write!(f, "Date"),
+        }
+    }
+}
+
+impl fmt::Debug for FieldType {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match *self {
+            TUnknown => write!(f, "Unknown"),
+            TNull => write!(f, "NULL"),
+            TString => write!(f, "String"),
             TFloat => write!(f, "Float"),
             TInteger => write!(f, "Integer"),
             TDate => write!(f, "Date"),
@@ -626,6 +646,7 @@ struct TypedSum {
 }
 
 impl TypedSum {
+    #[inline]
     fn add(&mut self, typ: FieldType, sample: &[u8]) {
         if sample.is_empty() {
             return;
@@ -655,7 +676,7 @@ impl TypedSum {
 
     fn show(&self, typ: FieldType) -> Option<String> {
         match typ {
-            TNull | TUnicode | TUnknown | TDate => None,
+            TNull | TString | TUnknown | TDate => None,
             TInteger => Some(self.integer.to_string()),
             TFloat => Some(self.float.unwrap_or(0.0).to_string()),
         }
@@ -685,6 +706,7 @@ struct TypedMinMax {
 }
 
 impl TypedMinMax {
+    #[inline]
     fn add(&mut self, typ: FieldType, sample: &[u8]) {
         self.str_len.add(sample.len());
         if sample.is_empty() {
@@ -692,7 +714,7 @@ impl TypedMinMax {
         }
         self.strings.add(sample.to_vec());
         match typ {
-            TUnicode | TUnknown | TNull => {}
+            TString | TUnknown | TNull => {}
             TFloat => {
                 let n = str::from_utf8(&*sample)
                     .ok()
@@ -730,7 +752,7 @@ impl TypedMinMax {
     fn show(&self, typ: FieldType) -> Option<(String, String)> {
         match typ {
             TNull => None,
-            TUnicode | TUnknown => match (self.strings.min(), self.strings.max()) {
+            TString | TUnknown => match (self.strings.min(), self.strings.max()) {
                 (Some(min), Some(max)) => {
                     let min = String::from_utf8_lossy(&**min).to_string();
                     let max = String::from_utf8_lossy(&**max).to_string();
@@ -764,6 +786,7 @@ impl Commute for TypedMinMax {
     }
 }
 
+#[inline]
 fn from_bytes<T: FromStr>(bytes: &[u8]) -> Option<T> {
     str::from_utf8(bytes).ok().and_then(|s| s.parse().ok())
 }
