@@ -15,7 +15,7 @@ use crate::CliResult;
 // rdr default is 8k in csv crate, we're doubling it
 const DEFAULT_RDR_BUFFER_CAPACITY: usize = 16 * (1 << 10);
 // previous wtr default in xsv is 32k, we're doubling it
-const DEFAULT_WTR_BUFFER_CAPACITY: usize = 64 * (1 << 10);
+pub const DEFAULT_WTR_BUFFER_CAPACITY: usize = 64 * (1 << 10);
 
 #[derive(Clone, Copy, Debug)]
 pub struct Delimiter(pub u8);
@@ -36,24 +36,16 @@ impl Delimiter {
         }
 
         if s.len() != 1 {
-            let msg = format!(
-                "Could not convert '{}' to a single \
-                            ASCII character.",
-                s
-            );
-            return Err(msg);
+            return Err(format!(
+                "Could not convert '{s}' to a single ASCII character."
+            ));
         }
 
         let c = s.chars().next().unwrap();
         if c.is_ascii() {
             Ok(Delimiter(c as u8))
         } else {
-            let msg = format!(
-                "Could not convert '{}' \
-                            to ASCII delimiter.",
-                c
-            );
-            Err(msg)
+            Err(format!("Could not convert '{c}' to ASCII delimiter."))
         }
     }
 }
